#include "leaderboard.h"
#include "PF.h"
#include "lcd_draw.h"

extern buzzer_menu_change_sound();

Leaderboard::Leaderboard() {
  start_idx = 0;
  entries = LEADERBOARD_ENTRIES;
  entries_per_page = LEADERBOARD_ENTRIES_PER_PAGE;
}

void Leaderboard::getScores() {
  int offset = 0;
  int bytes_to_read = sizeof(scores);

  /* reads from the 10th sector (hardcoded) into the scores array */
  DRESULT res = disk_readp((BYTE *) scores, 10, 0, sizeof(scores));
  if (res) {
    Serial.println(F("Read failed"));
    while (1);
  }
}

void Leaderboard::addScore(int score) {
  getScores();

  bool updated = false;

  for (int i = 0; i < LEADERBOARD_ENTRIES; i++) {
    if (score > scores[i]) {
      for (int j = LEADERBOARD_ENTRIES - 1; j > i; j--)
        scores[j] = scores[j - 1];

      scores[i] = score;
      updated = true;
      break;
    }
  }

  /* If the leaderboard was updated, write it back to the sd card */
  if (updated) {
    disk_writep(0, 10);
    disk_writep((BYTE *) scores, sizeof(scores));
    disk_writep(0, 0);
  }
}

void Leaderboard::resetAttributes() {
  start_idx = 0;
}

void Leaderboard::pageUp() {
  if (start_idx > 0) {
    start_idx -= entries_per_page;
    buzzer_menu_change_sound();
    draw_leaderboard(*this);
  }
}

void Leaderboard::pageDown() {
  if (start_idx < entries - entries_per_page) {
      start_idx += entries_per_page;
      buzzer_menu_change_sound();
      draw_leaderboard(*this);
    }
}
