#include "multiplayer.h"
#include "lcd_draw.h"
#include "leaderboard.h"
#include "apds.h"

extern void back_to_main_menu();
extern volatile bool button_select_pressed;

extern buzzer_correct_sound();
extern buzzer_incorrect_sound();
extern buzzer_game_over_sound();
extern buzzer_new_game_sound();

Multiplayer::Multiplayer() {
  started = false;

  time_left = 11;
  game_level = 0;
  
  lives[0] = 3;
  lives[1] = 3;
  
  is_game_over = false;

  winner = 1;
  player2_input_idx = 0;

  display_changed = false;
}

void Multiplayer::handlePlayer1Gesture() {
  /* Player1 creates the sequence */
  int gesture = get_gesture();

  if (gesture < 1)
    return;

  generated_sequence[player1_input_idx] = gesture;
  player1_input_idx++;
  buzzer_correct_sound();
  
  draw_direction(gesture);
  delay(1000);
}

void Multiplayer::handlePlayer2Gesture(int player2_idx) {
  /* Player2 has to reproduce the sequence */
  int gesture = get_gesture();

  if (gesture < 1)
    return;

  if (generated_sequence[player2_input_idx] == gesture) {
    buzzer_correct_sound();
    
    player2_input_idx++;
  } else {
    lives[player2_idx]--;
    display_changed = true;
  
    if (lives[player2_idx] == 0) {
      is_game_over = true;
    } else {
      buzzer_incorrect_sound();
    }
  }
}

void Multiplayer::start(int start_time, int start_lives) {
  started = true;

  lives[0] = start_lives;
  lives[1] = start_lives;

  player1_input_idx = 0;
  player2_input_idx = 0;

  play(start_time);
}

void Multiplayer::handleTimerInterrupt() {
  time_left--;
  display_changed = true;
    
  if (time_left == 0) {
    is_game_over = true;
  }
}

void Multiplayer::playTurn(int turn) {
  /* Player 1 generates the sequence */
  enable_gestures();
  
  while (player1_input_idx <= game_level) {
    display_player1_screen(turn, game_level - player1_input_idx + 1);
     
    while(gesture_interrupt_flag == 0);
     
    detachInterrupt(0);
    handlePlayer1Gesture();
    gesture_interrupt_flag = 0;
    attachInterrupt(0, gestureInterrupt, FALLING);
  }
  
  disable_gestures();

  display_start_level();

  enable_gestures();

  /* Player 2 has to reproduce the sequence */
  cli();
  TCNT1H = 0;
  TCNT1L = 0;
  OCR1A = 15624;
  TIMSK1 |= (1 << OCIE1A); // start timer1
  sei();

  // loop until timer expires or lives become 0
  while (!is_game_over) {
    // finished level
    if (player2_input_idx == game_level + 1) {
      TIMSK1 &= ~(1 << OCIE1A);
      break;
    }

    if(gesture_interrupt_flag == 1) {
      detachInterrupt(0);
      handlePlayer2Gesture((turn - 1 + 1) % 2);
      gesture_interrupt_flag = 0;
      attachInterrupt(0, gestureInterrupt, FALLING);
    }
    
    if (display_changed) {
      display_player2_screen(time_left, lives[(turn - 1 + 1) % 2], turn);
    }
  }

  disable_gestures();
}
  
void Multiplayer::play(int start_time) {
  int turn;

  /* Loop until one of the players loses
   * play two turns at each game level
   */
  while (!is_game_over) {
    time_left = start_time + game_level * 3;
    turn = 1;
    playTurn(turn);

    if (!is_game_over) {
      player1_input_idx = 0;
      player2_input_idx = 0;
    } else {
      winner = turn;
      break;
    }

    time_left = start_time + game_level * 3;
    turn = 2;
    playTurn(turn);

    if (!is_game_over) {
      player1_input_idx = 0;
      player2_input_idx = 0;
      game_level++;
    } else {
      winner = turn;
    }
  }

  finishGame();
}

void Multiplayer::finishGame() {
  cli();
  TIMSK1 &= ~(1 << OCIE1A);
  sei();

  draw_game_over(winner, false);
  
  buzzer_game_over_sound();
  
  while (true) {
    if (button_select_pressed) {
      back_to_main_menu();
      
      buzzer_new_game_sound();
      
      button_select_pressed = false;
      break;
    }
  }

  is_game_over = false;
  resetAttributes();
}

void Multiplayer::resetAttributes() {
  started = false;

  time_left = 11;
  lives[0] = 3;
  lives[1] = 3;
  game_level = 0;
    
  is_game_over = false;

  player1_input_idx = 0;
  player2_input_idx = 0;

  display_changed = false;
}
