#include "singleplayer.h"
#include "lcd_draw.h"
#include "leaderboard.h"
#include "apds.h"

extern void back_to_main_menu();
extern volatile bool button_select_pressed;

extern buzzer_correct_sound();
extern buzzer_incorrect_sound();
extern buzzer_game_over_sound();
extern buzzer_new_game_sound();

Singleplayer::Singleplayer(Leaderboard leaderboard) : leaderboard(leaderboard) {
  started = false;

  time_left = 11;
  lives = 3;
  game_level = 0;
  score = 0;
    
  is_game_over = false;

  user_input_idx = 0;

  display_changed = false;
}

void Singleplayer::resetAttributes() {
  started = false;

  time_left = 11;
  lives = 3;
  game_level = 0;
  score = 0;
    
  is_game_over = false;

  user_input_idx = 0;

  display_changed = false;
}

void Singleplayer::handleGesture() {
  int gesture = get_gesture();

  if (gesture < 1)
    return;

  if (generated_sequence[user_input_idx] == gesture) {
    buzzer_correct_sound();
    
    user_input_idx++;
  } else {
    lives--;
    display_changed = true;
  
    if (lives == 0) {
      is_game_over = true;
    } else {
      buzzer_incorrect_sound();
    }
  }
}

void Singleplayer::start(int start_time, int start_lives) {
  started = true;

  time_left = start_time;
  lives = start_lives;
  score = 0;

  play(start_time);
}

void Singleplayer::handleTimerInterrupt() {
  time_left--;
  display_changed = true;
    
  if (time_left == 0) {
    is_game_over = true;
  }
}

void Singleplayer::initialiseLevel(int start_time) {
  for (int i = 0; i <= game_level; i++) {
    generated_sequence[i] = 1 + random(4);
  }
  
  time_left = start_time + game_level * 3;
  user_input_idx = 0;
}

void Singleplayer::displaySequence() {
  for (int i = 0; i <= game_level; i++) {
    draw_direction(generated_sequence[i]);
    delay(900);
    
    clear_display();
    delay(100);
  }
}

void Singleplayer::playLevel(int start_time) {
  initialiseLevel(start_time);
  displaySequence();
  
  draw_singleplayer(score, time_left, lives);

  enable_gestures();

  cli();
  TCNT1H = 0;
  TCNT1L = 0;
  OCR1A = 15624;
  TIMSK1 |= (1 << OCIE1A); // start timer1
  sei();

  // loop until timer expires or lives become 0
  while (!is_game_over) {
    // finished level
    if (user_input_idx == game_level + 1) {
      TIMSK1 &= ~(1 << OCIE1A);
      break;
    }

    if(gesture_interrupt_flag == 1) {
      detachInterrupt(0);
      handleGesture();
      gesture_interrupt_flag = 0;
      attachInterrupt(0, gestureInterrupt, FALLING);
    }
    
    if (display_changed) {
      draw_singleplayer(score, time_left, lives);
    }
  }

  disable_gestures();
}

void Singleplayer::play(int start_time) {
  while (!is_game_over) {
    display_start_level();
    
    playLevel(start_time);

    if (!is_game_over) {
      user_input_idx = 0;
      game_level++;
      score++;
    }
  }

  finishGame();
}

void Singleplayer::finishGame() {
  cli();
  TIMSK1 &= ~(1 << OCIE1A);
  sei();

  draw_game_over(score, true);
  
  buzzer_game_over_sound();
  
  leaderboard.addScore(score);
  
  while (true) {
    if (button_select_pressed) {
      back_to_main_menu();
      
      buzzer_new_game_sound();
      
      button_select_pressed = false;
      break;
    }
  }

  is_game_over = false;
  resetAttributes();
}
